@@ -19,17 +19,17 @@
 */
 
 #include "booledit.h"
+#include "KPropertyListData.h"
 #include "KPropertyUtils.h"
+#include "KPropertyUtils_p.h"
 #include "kproperty_debug.h"
 
 #include <QIcon>
-#include <QApplication>
-#include <QPainter>
 #include <QVariant>
 
 /*! @return name for state with index @a index,
  where 0 means true, 1 means false and 2 means none */
-static QString stateName(int index, const QLocale &locale, const KProperty* prop = 0)
+static QString stateName(int index, const QLocale &locale, const KProperty* prop = nullptr)
 {
     QString stateNameString;
     if (index == 0) {
@@ -57,10 +57,11 @@ static QString stateName(int index, const QLocale &locale, const KProperty* prop
 }
 
 //! Sets up @a data list data with keys and names for true, false, none values, respectively
-static void setupThreeStateListData(KPropertyListData &data, const KProperty* prop)
+static void setupThreeStateListData(KPropertyListData *data, const KProperty* prop)
 {
-    data.keys << true << false << QVariant();
-    data.names << stateName(0, QLocale(), prop) << stateName(1, QLocale(), prop) << stateName(2, QLocale(), prop);
+    data->setKeys({ true, false, QVariant() });
+    data->setNamesAsStringList({ stateName(0, QLocale(), prop), stateName(1, QLocale(), prop),
+                                 stateName(2, QLocale(), prop) });
 }
 
 static int valueToIndex(const QVariant& value)
@@ -94,7 +95,7 @@ public:
 
 Q_GLOBAL_STATIC(BoolEditGlobal, g_boolEdit)
 
-class KPropertyBoolEditor::Private
+class Q_DECL_HIDDEN KPropertyBoolEditor::Private
 {
 public:
     Private(const KProperty *prop)
@@ -112,27 +113,7 @@ KPropertyBoolEditor::KPropertyBoolEditor(const KProperty *prop, QWidget *parent)
 {
     setFocusPolicy(Qt::WheelFocus);
     setCheckable(true);
-//    setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     setAutoFillBackground(true);
-//    setFlat(false);
-//    setStyle(qApp->style());
-//    setPalette(qApp->palette());
-//    setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
-    //we're not using layout to because of problems with button size
-//    m_toggle->move(0, 0);
-//    m_toggle->resize(width(), height());
-/*    KColorScheme cs(QPalette::Active);
-    QColor focus = cs.decoration(KColorScheme::FocusColor).color();
-    setStyleSheet(
-        QString::fromLatin1("QToolButton { "
-        "border: 1px solid %1; "
-        "border-radius: 0px; "
-        "padding: 0 0px; }").arg(focus.name())); */
-
-//    setFocusWidget(m_toggle);
-//    setStyleSheet(
-//        QLatin1String(" QPushButton { border: none; padding:0;background-color: red; }") );
-
     connect(this, SIGNAL(toggled(bool)), this, SLOT(slotValueChanged(bool)));
 }
 
@@ -148,15 +129,10 @@ QVariant KPropertyBoolEditor::value() const
 
 void KPropertyBoolEditor::setValue(const QVariant &value)
 {
-//    m_toggle->blockSignals(true);
     d->value = value;
     if (value.type() == QVariant::Bool) {
         setChecked(value.toBool());
     }
-//    setState(value);
-//    m_toggle->blockSignals(false);
-//    if (emitChange)
-//        emit valueChanged(this);
 }
 
 void
@@ -164,8 +140,6 @@ KPropertyBoolEditor::slotValueChanged(bool state)
 {
     d->value = state;
     emit commitData(this);
-//    setState(state);
-////    emit valueChanged(this);
 }
 
 void KPropertyBoolEditor::draw(QPainter *p, const QRect &r, const QVariant &value,
@@ -214,27 +188,20 @@ bool KPropertyBoolEditor::eventFilter(QObject* watched, QEvent* e)
         QKeyEvent* ev = static_cast<QKeyEvent*>(e);
         const int k = ev->key();
         if (k == Qt::Key_Space || k == Qt::Key_Enter || k == Qt::Key_Return) {
-//            if (m_toggle)
-                toggle();
+            toggle();
             return true;
         }
     }
     return QToolButton::eventFilter(watched, e);
 }
 
-/*void
-BoolEdit::setReadOnlyInternal(bool readOnly)
-{
-    setVisibleFlag(!readOnly);
-}*/
-
 //--------------------------------------------------
 
-class ThreeStateBoolIconProvider : public KPropertyComboBoxEditor::Options::IconProviderInterface
+class ThreeStateBoolIconProvider : public KPropertyComboBoxEditorIconProviderInterface
 {
 public:
     ThreeStateBoolIconProvider() {}
-    virtual QIcon icon(int index) const
+    QIcon icon(int index) const override
     {
           if (index == 0)
               return g_boolEdit->yesIcon;
@@ -242,23 +209,29 @@ public:
               return g_boolEdit->noIcon;
           return g_boolEdit->noneIcon;
     }
-    virtual IconProviderInterface* clone() const
+    KPropertyComboBoxEditorIconProviderInterface* clone() const override
     {
         return new ThreeStateBoolIconProvider();
     }
 };
 
-static KPropertyComboBoxEditor::Options initThreeStateBoolOptions()
+static KPropertyComboBoxEditorOptions initThreeStateBoolOptions()
 {
-    KPropertyComboBoxEditor::Options options;
+    KPropertyComboBoxEditorOptions options;
     options.iconProvider = new ThreeStateBoolIconProvider();
     return options;
 }
 
-KPropertyThreeStateBoolEditor::KPropertyThreeStateBoolEditor(
-    const KPropertyListData& listData,
-    QWidget *parent)
-        : KPropertyComboBoxEditor(listData, initThreeStateBoolOptions(), parent)
+class Q_DECL_HIDDEN KPropertyThreeStateBoolEditor::Private
+{
+public:
+    Private() {
+    }
+};
+
+KPropertyThreeStateBoolEditor::KPropertyThreeStateBoolEditor(const KPropertyListData &listData,
+                                                             QWidget *parent)
+    : KPropertyComboBoxEditor(listData, initThreeStateBoolOptions(), parent), d(new Private)
 {
 //    QPixmap nullIcon(m_yesIcon.size());   //transparent pixmap of appropriate size
 //    nullIcon.fill(Qt::transparent);
@@ -268,6 +241,7 @@ KPropertyThreeStateBoolEditor::KPropertyThreeStateBoolEditor(
 
 KPropertyThreeStateBoolEditor::~KPropertyThreeStateBoolEditor()
 {
+    delete d;
 }
 
 QVariant KPropertyThreeStateBoolEditor::value() const
@@ -286,25 +260,19 @@ QVariant KPropertyThreeStateBoolEditor::value() const
     Widget::setProperty(prop);
     m_setValueEnabled = true;
     if (prop)
-        setValue(prop->value(), false); //now the value can be set
+        setValue(prop->value(), KProperty::ValueOption::IgnoreOld); //now the value can be set
 }*/
 
 void KPropertyThreeStateBoolEditor::setValue(const QVariant &value)
 {
-//    if (!m_setValueEnabled)
-//        return;
-
     setCurrentIndex( valueToIndex(value) );
-
-//    if (emitChange)
-//        emit valueChanged(this);
 }
 
 //---------------
 
 KPropertyBoolDelegate::KPropertyBoolDelegate()
 {
-    options.removeBorders = false;
+    options()->setBordersVisible(true);
 }
 
 QWidget * KPropertyBoolDelegate::createEditor( int type, QWidget *parent,
@@ -315,9 +283,9 @@ QWidget * KPropertyBoolDelegate::createEditor( int type, QWidget *parent,
     KProperty *prop = KPropertyUtils::propertyForIndex(index);
 
     // boolean editors can optionally accept 3rd state:
-    if (prop->option("3State", false).toBool()) {
+    if (prop && prop->option("3State", false).toBool()) {
         KPropertyListData threeStateListData;
-        setupThreeStateListData(threeStateListData, prop);
+        setupThreeStateListData(&threeStateListData, prop);
         return new KPropertyThreeStateBoolEditor(threeStateListData, parent);
     }
     else {
@@ -328,7 +296,7 @@ QWidget * KPropertyBoolDelegate::createEditor( int type, QWidget *parent,
 void KPropertyBoolDelegate::paint( QPainter * painter,
     const QStyleOptionViewItem & option, const QModelIndex & index ) const
 {
-    painter->save();
+    const KPropertyUtilsPrivate::PainterSaver saver(painter);
     KProperty *prop = KPropertyUtils::propertyForIndex(index);
     if (!prop) {
         return;
@@ -337,7 +305,6 @@ void KPropertyBoolDelegate::paint( QPainter * painter,
     QRect rect(option.rect);
     const bool threeState = prop->option("3State", false).toBool();
     KPropertyBoolEditor::draw(painter, rect.translated(0, -2), value, propertyValueToString(prop, QLocale()), threeState);
-    painter->restore();
 }
 
 QString KPropertyBoolDelegate::propertyValueToString(const KProperty* prop, const QLocale &locale) const
