@@ -22,6 +22,7 @@
 #include "cursoredit.h"
 #include "KProperty.h"
 #include "KPropertyCoreUtils_p.h"
+#include "KPropertyListData.h"
 #include "KPropertyUtils_p.h"
 
 #include "xpm/blank_cursor.xpm"
@@ -56,15 +57,14 @@ public:
 
     Qt::CursorShape indexToShape(int index) const
     {
-        if (index < 0 || index >= keys.count())
-            return Qt::ArrowCursor;
-        return (Qt::CursorShape)(keys[index].toInt());
+        return static_cast<Qt::CursorShape>(
+            keys().value(index, static_cast<int>(Qt::ArrowCursor)).toInt());
     }
 
     int shapeToIndex(Qt::CursorShape _shape) const
     {
         int index = 0;
-        foreach (const QVariant& shape, keys) {
+        for (const QVariant &shape : keys()) {
             if (shape.toInt() == _shape)
                 return index;
             index++;
@@ -165,7 +165,8 @@ const char * const * const CursorListData::m_xpms[] =
 Q_GLOBAL_STATIC(CursorListData, s_cursorListData)
 
 //----------------------
-class CursorIconProvider : public KPropertyComboBoxEditor::Options::IconProviderInterface
+
+class CursorIconProvider : public KPropertyComboBoxEditorIconProviderInterface
 {
 public:
     explicit CursorIconProvider(QWidget* parent) : m_parent(parent) {}
@@ -173,7 +174,7 @@ public:
     {
           return s_cursorListData->pixmapForIndex(index, m_parent->palette());
     }
-    IconProviderInterface *clone() const override
+    KPropertyComboBoxEditorIconProviderInterface *clone() const override
     {
         return new CursorIconProvider(m_parent);
     }
@@ -182,20 +183,29 @@ public:
 
 //----------------------
 
-static KPropertyComboBoxEditor::Options initComboBoxOptions(QWidget* parent)
+static KPropertyComboBoxEditorOptions initComboBoxOptions(QWidget* parent)
 {
-    KPropertyComboBoxEditor::Options options;
+    KPropertyComboBoxEditorOptions options;
     options.iconProvider = new CursorIconProvider(parent);
     return options;
 }
 
+class Q_DECL_HIDDEN KPropertyCursorEditor::Private
+{
+public:
+    Private()
+    {
+    }
+};
+
 KPropertyCursorEditor::KPropertyCursorEditor(QWidget *parent)
-        : KPropertyComboBoxEditor(*s_cursorListData, initComboBoxOptions( this ), parent)
+        : KPropertyComboBoxEditor(*s_cursorListData, initComboBoxOptions( this ), parent),
+          d(new Private)
 {
     int paddingTop = 1;
     int paddingLeft = 2;
     const QString style(parent->style()->objectName());
-    if (!KPropertyUtils::gridLineColor(this).isValid()) {
+    if (!KPropertyUtilsPrivate::gridLineColor(this).isValid()) {
         setFrame(false);
         paddingTop = 0;
     }
@@ -211,6 +221,7 @@ KPropertyCursorEditor::KPropertyCursorEditor(QWidget *parent)
 
 KPropertyCursorEditor::~KPropertyCursorEditor()
 {
+    delete d;
 }
 
 QCursor KPropertyCursorEditor::cursorValue() const
@@ -227,7 +238,7 @@ void KPropertyCursorEditor::setCursorValue(const QCursor &value)
 
 KPropertyCursorDelegate::KPropertyCursorDelegate()
 {
-    options.removeBorders = false;
+    options()->setBordersVisible(true);
 }
 
 QWidget * KPropertyCursorDelegate::createEditor( int type, QWidget *parent,
@@ -242,7 +253,7 @@ QWidget * KPropertyCursorDelegate::createEditor( int type, QWidget *parent,
 void KPropertyCursorDelegate::paint( QPainter * painter,
     const QStyleOptionViewItem & option, const QModelIndex & index ) const
 {
-    const KPropertyUtils::PainterSaver saver(painter);
+    const KPropertyUtilsPrivate::PainterSaver saver(painter);
     int comboIndex = s_cursorListData->shapeToIndex( index.data(Qt::EditRole).value<QCursor>().shape() );
     int pmSize = (option.rect.height() >= 32) ? 32 : 16;
     const QPixmap pm( s_cursorListData->pixmapForIndex(comboIndex, option.palette)
@@ -261,5 +272,5 @@ QString KPropertyCursorDelegate::valueToString(const QVariant& value, const QLoc
         return KPropertyUtils::keyForEnumValue("CursorShape", shape);
     }
     const int comboIndex = s_cursorListData->shapeToIndex(shape);
-    return s_cursorListData->names[comboIndex];
+    return s_cursorListData->names().value(comboIndex).toString();
 }
